# Copyright 2021- Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)
import base64
import csv
import io
from datetime import datetime, timedelta

from dateutil.relativedelta import relativedelta

from odoo import fields, models

from odoo.addons.api_connector.tools.date_utils import local_to_utc


class AccOperationImportWizard(models.TransientModel):
    _name = "acc.operation.import.wizard"
    _description = "Wizard: Import des données"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    operation_id = fields.Many2one("acc.operation", "Opération liée")
    message = fields.Text(
        string="Message Log",
    )
    model = fields.Selection(selection=[], required=True)
    attachment_ids = fields.Many2many(
        comodel_name="ir.attachment", string="Documents à importer"
    )

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
    def delete_existing_data(self, data_filename, counter_id):
        message = ""
        # Suppression des données si déjà existantes
        date_begin_str = data_filename[1]
        date_end_str = data_filename[2]
        date_format = "%d%m%Y"
        date_begin_obj = datetime.strptime(date_begin_str, date_format)
        date_end_obj = datetime.strptime(date_end_str, date_format) + relativedelta(
            days=1
        )

        # Vérification si des données existent déjà
        # pour cet intervalle de date
        start_datetime = local_to_utc(date_begin_obj, "Europe/Paris")
        end_datetime = local_to_utc(date_end_obj, "Europe/Paris")

        computed_data_type = data_filename[3].lower()
        if computed_data_type == "conso":
            computed_data_type = "cons"
        elif computed_data_type == "autoconso":
            computed_data_type = "autocons"

        domain_all = [
            ("date_slot", ">=", start_datetime),
            ("date_slot", "<", end_datetime),
            ("acc_operation_id", "=", self.operation_id.id),
            ("comp_data_type", "=", computed_data_type),
            ("acc_counter_id", "=", counter_id.id),
        ]
        rec_ids = self.env[self.model].search(domain_all)
        if rec_ids:
            message += "Suppression des enregistrements existants ... <br/>"
            rec_ids.unlink()
            message += "Suppression des enregistrements existants OK <br/>"

        return message

    def create_curve(self, curve_data):
        self.env[self.model].create(curve_data)

    def update_partner_id(self, data_filename, counter_id):
        date_begin_str = data_filename[1]
        date_end_str = data_filename[2]
        date_format = "%d%m%Y"
        date_begin_obj = datetime.strptime(date_begin_str, date_format)
        date_end_obj = datetime.strptime(date_end_str, date_format) + relativedelta(
            days=1
        )
        # Update partner_id for retrieved cdc
        domain = [
            ("acc_operation_id", "=", self.operation_id.id),
            ("acc_counter_id", "=", counter_id.id or False),
            (
                "prm_type",
                "=",
                "delivery" if data_filename[3] in ("autocons", "cons") else "injection",
            ),
        ]
        self.env["acc.counter.period"]._get_periods_from_interval(
            domain, date_begin_obj, date_end_obj
        )._update_cdc_partner_id(model=self.model)

    def valid_import(self):
        message = ""
        message += (
            "<h1>Début Import manuel: "
            + str(fields.Datetime.now())
            + " - "
            + self.model
            + "</h1>"
        )
        for file in self.attachment_ids:
            message += (
                "<p><strong>Fichier "
                + file.name
                + "</strong><br/>Début Import ... <br/>"
            )
            data_filename = file.name.split("_")
            id_pdm = data_filename[0]

            counter_id = self.env["acc.counter"].search([("name", "=", id_pdm)])
            data_filename[3] = data_filename[3].lower()

            if data_filename[3] not in ["prod", "surplus"]:
                data_filename[3] = data_filename[3][:-1]

            message += self.delete_existing_data(
                data_filename=data_filename, counter_id=counter_id
            )

            file_decode = io.StringIO(base64.b64decode(file.datas).decode("UTF-8"))
            file_decode.seek(0)

            file_reader = []
            csv_reader = csv.reader(file_decode, delimiter=";")
            file_reader.extend(csv_reader)

            # Create Data for the CDC
            message += "Lecture et import des données ... <br/>"
            for row in file_reader:
                power_data = row[1:]
                # check case of csv line ending by delimiter
                if power_data[-1] == "":
                    power_data.pop()

                slot_datetime_tz = datetime.strptime(row[0], "%d/%m/%Y %H:%M")
                slot_datetime_utc = local_to_utc(slot_datetime_tz, "Europe/Paris")

                # if 2 data it's a 30 min step file if 4 it's a 15 min one
                if len(power_data) == 4:
                    timestep = 15
                else:
                    timestep = 30

                for index, power in enumerate(power_data):
                    if index == 0:
                        timestamp = slot_datetime_utc
                    else:
                        timestamp = timestamp + timedelta(minutes=timestep)

                    self.create_curve(
                        {
                            "name": file.name,
                            "acc_operation_id": self.operation_id.id,
                            "acc_counter_id": counter_id.id,
                            "comp_data_type": data_filename[3],
                            "power": power,
                            "date_slot": timestamp,
                            "timestep": timestep,
                        }
                    )

            self.update_partner_id(data_filename, counter_id)

            message += "Fin de l'import des données OK<br/>"
            # Suppression du fichier après création des enregistrements
            message += "Suppression du fichier " + file.name + " ...<br/>"
            file.unlink()
            message += "Suppression OK </p>"

        message += "<h1>Fin import manuel: " + str(fields.Datetime.now()) + "</h1>"
        # Logs information logs
        log_id = self.env["acc.logs"].create(
            {
                "name": "Import du "
                + str(fields.Date.today())
                + " manuel - "
                + self.model,
                "date_launched": fields.Datetime.now(),
                "type_log": "manual",
                "message": message,
                "acc_operation_id": self.operation_id.id,
            }
        )

        view_id = self.env.ref("oacc.acc_logs_form").id
        return {
            "name": "LOGS",
            "view_type": "form",
            "view_mode": "form",
            "views": [(view_id, "form")],
            "res_model": "acc.logs",
            "view_id": view_id,
            "type": "ir.actions.act_window",
            "res_id": log_id.id,
            "target": "new",
            "flags": {"initial_mode": "view"},
        }
